@@ -266,8 +266,8 @@ struct MHD_Response
   char *data;
 
   /**
-   * Closure to give to the content reader
-   * free callback.
+   * Closure to give to the content reader @e crc
+   * and content reader free callback @e crfc.
    */
   void *crc_cls;
 
@@ -284,8 +284,8 @@ struct MHD_Response
   MHD_ContentReaderFreeCallback crfc;
 
   /**
-   * Mutex to synchronize access to data/size and
-   * reference counts.
+   * Mutex to synchronize access to @e data, @e size and
+   * @e reference_count.
    */
   MHD_mutex_ mutex;
 
@@ -296,22 +296,23 @@ struct MHD_Response
 
   /**
    * At what offset in the stream is the
-   * beginning of data located?
+   * beginning of @e data located?
    */
   uint64_t data_start;
 
   /**
-   * Offset to start reading from when using 'fd'.
+   * Offset to start reading from when using @e fd.
    */
   off_t fd_off;
 
   /**
-   * Size of data.
+   * Number of bytes ready in @e data (buffer may be larger
+   * than what is filled with payload).
    */
   size_t data_size;
 
   /**
-   * Size of the data buffer.
+   * Size of the data buffer @e data.
    */
   size_t data_buffer_size;
 
@@ -326,6 +327,11 @@ struct MHD_Response
    */
   int fd;
 
+  /**
+   * Flags set for the MHD response.
+   */
+  enum MHD_ResponseFlags flags;
+
 };
 
 
@@ -535,6 +541,10 @@ struct MHD_Connection
 
   /**
    * Next pointer for the XDLL organizing connections by timeout.
+   * This DLL can be either the
+   * 'manual_timeout_head/manual_timeout_tail' or the
+   * 'normal_timeout_head/normal_timeout_tail', depending on whether a
+   * custom timeout is set for the connection.
    */
   struct MHD_Connection *nextX;
 
@@ -716,7 +726,7 @@ struct MHD_Connection
   int client_aware;
 
   /**
-   * Socket for this connection.  Set to MHD_INVALID_SOCKET if
+   * Socket for this connection.  Set to #MHD_INVALID_SOCKET if
    * this connection has died (daemon should clean
    * up in that case).
    */
@@ -731,7 +741,7 @@ struct MHD_Connection
   int read_closed;
 
   /**
-   * Set to MHD_YES if the thread has been joined.
+   * Set to #MHD_YES if the thread has been joined.
    */
   int thread_joined;
 
@@ -868,7 +878,7 @@ typedef void * (*LogCallback)(void * cls,
 
 /**
  * Signature of function called to unescape URIs.  See also
- * MHD_http_unescape.
+ * #MHD_http_unescape().
  *
  * @param cls closure
  * @param conn connection handle
@@ -1007,7 +1017,7 @@ struct MHD_Daemon
   LogCallback uri_log_callback;
 
   /**
-   * Closure argument to uri_log_callback.
+   * Closure argument to @e uri_log_callback.
    */
   void *uri_log_callback_cls;
 
@@ -1017,7 +1027,7 @@ struct MHD_Daemon
   UnescapeCallback unescape_callback;
 
   /**
-   * Closure for unescape callback.
+   * Closure for @e unescape_callback.
    */
   void *unescape_callback_cls;
 
@@ -1252,7 +1262,7 @@ struct MHD_Daemon
 
 
 #if EXTRA_CHECKS
-#define EXTRA_CHECK(a) if (!(a)) abort();
+#define EXTRA_CHECK(a) do { if (!(a)) abort(); } while (0)
 #else
 #define EXTRA_CHECK(a)
 #endif
@@ -1267,6 +1277,8 @@ struct MHD_Daemon
  * @param element element to insert
  */
 #define DLL_insert(head,tail,element) do { \
+  EXTRA_CHECK (NULL == (element)->next); \
+  EXTRA_CHECK (NULL == (element)->prev); \
   (element)->next = (head); \
   (element)->prev = NULL; \
   if ((tail) == NULL) \
@@ -1286,6 +1298,8 @@ struct MHD_Daemon
  * @param element element to remove
  */
 #define DLL_remove(head,tail,element) do { \
+  EXTRA_CHECK ( (NULL != (element)->next) || ((element) == (tail)));  \
+  EXTRA_CHECK ( (NULL != (element)->prev) || ((element) == (head)));  \
   if ((element)->prev == NULL) \
     (head) = (element)->next;  \
   else \
@@ -1308,9 +1322,11 @@ struct MHD_Daemon
  * @param element element to insert
  */
 #define XDLL_insert(head,tail,element) do { \
+  EXTRA_CHECK (NULL == (element)->nextX); \
+  EXTRA_CHECK (NULL == (element)->prevX); \
   (element)->nextX = (head); \
   (element)->prevX = NULL; \
-  if ((tail) == NULL) \
+  if (NULL == (tail)) \
     (tail) = element; \
   else \
     (head)->prevX = element; \
@@ -1327,11 +1343,13 @@ struct MHD_Daemon
  * @param element element to remove
  */
 #define XDLL_remove(head,tail,element) do { \
-  if ((element)->prevX == NULL) \
+  EXTRA_CHECK ( (NULL != (element)->nextX) || ((element) == (tail)));  \
+  EXTRA_CHECK ( (NULL != (element)->prevX) || ((element) == (head)));  \
+  if (NULL == (element)->prevX) \
     (head) = (element)->nextX;  \
   else \
     (element)->prevX->nextX = (element)->nextX; \
-  if ((element)->nextX == NULL) \
+  if (NULL == (element)->nextX) \
     (tail) = (element)->prevX;  \
   else \
     (element)->nextX->prevX = (element)->prevX; \
@@ -1380,12 +1398,23 @@ struct MHD_Daemon
 
 
 /**
- * Equivalent to time(NULL) but tries to use some sort of monotonic
+ * Equivalent to `time(NULL)` but tries to use some sort of monotonic
  * clock that isn't affected by someone setting the system real time
  * clock.
  *
  * @return 'current' time
  */
-time_t MHD_monotonic_time(void);
+time_t
+MHD_monotonic_time(void);
+
+
+/**
+ * Convert all occurences of '+' to ' '.
+ *
+ * @param arg string that is modified (in place), must be 0-terminated
+ */
+void
+MHD_unescape_plus (char *arg);
+
 
 #endif
